'use strict';

const {
  getElementsByTagName,
  createDocumentFragment,
  createTextNode,
  createElement,
  normalize
} = document;
const specificWords = /* @__PURE__ */ new Set([
  "the",
  "and",
  "in",
  "on",
  "at",
  "by",
  "with",
  "about",
  "against",
  "between",
  "into",
  "through",
  "during",
  "before",
  "after",
  "above",
  "below",
  "to",
  "from",
  "up",
  "down",
  "over",
  "under",
  "again",
  "further",
  "then",
  "once",
  "here",
  "there",
  "when",
  "where",
  "why",
  "how",
  "all",
  "any",
  "both",
  "each",
  "few",
  "more",
  "most",
  "other",
  "some"
]);
function isSpecificWord(word) {
  return specificWords.has(word.toLowerCase());
}
const skippedTags = /* @__PURE__ */ new Set([
  "PRE",
  "CODE",
  "SCRIPT",
  "STYLE",
  "TITLE",
  "HEAD",
  "META",
  "LINK",
  "NOSCRIPT",
  "TEMPLATE",
  "SVG",
  "CANVAS",
  "SLOT"
]);

function highlightPrefixLength(word) {
  if (isSpecificWord(word)) {
    return 1;
  } else if (word.length <= 2) {
    return 1;
  } else if (word.length <= 5) {
    return 2;
  } else {
    return Math.ceil(word.length / 2);
  }
}
function highlightSentence(textNode, calculatePrefixLength) {
  const content = textNode.textContent;
  if (/^\s+$/.test(content)) {
    return [createTextNode.call(textNode.ownerDocument, content)];
  }
  const nodesList = [];
  const words = content.split(/(\s+|[.,!?;()])/).filter(Boolean);
  for (const word of words) {
    if (/^[a-z]+$/i.test(word)) {
      const boldLength = calculatePrefixLength(word);
      const boldPart = createElement.call(textNode.ownerDocument, "b");
      boldPart.textContent = word.slice(0, boldLength);
      const restPart = createTextNode.call(textNode.ownerDocument, word.slice(boldLength));
      nodesList.push(boldPart, restPart);
    } else {
      const punctuation = createTextNode.call(textNode.ownerDocument, word);
      nodesList.push(punctuation);
    }
  }
  return nodesList;
}
function prehighlight(html, options = {}) {
  if (typeof html === "string" && options.inplace) {
    throw new Error("inplace option only works with Node type");
  }
  const calculatePrefixLength = options.highlightPrefixLength || highlightPrefixLength;
  const domParser = new DOMParser();
  let body;
  if (!options.inplace) {
    if (typeof html === "string") {
      const doc = domParser.parseFromString(html, "text/html");
      body = getElementsByTagName.call(doc, "body")[0];
    } else {
      const doc = domParser.parseFromString("<!---->", "text/html");
      body = getElementsByTagName.call(doc, "body")[0];
      const importedNode = body.ownerDocument.importNode(html, true);
      if (importedNode.nodeType === 1 && importedNode.nodeName === "BODY") {
        body = importedNode;
      } else if (importedNode.nodeName === "HTML") {
        body = getElementsByTagName.call(importedNode, "body")[0];
      } else {
        body.appendChild(importedNode);
      }
    }
  }
  const textNodes = [];
  function traverseDOM(node) {
    if (node.nodeType === Node.TEXT_NODE) {
      textNodes.push(node);
    } else if (node.nodeType === Node.ELEMENT_NODE) {
      if (!skippedTags.has(node.tagName)) {
        node.childNodes.forEach(traverseDOM);
      }
    }
  }
  traverseDOM(options.inplace ? html : body);
  textNodes.forEach((textNode) => {
    const highlightedNodes = highlightSentence(textNode, calculatePrefixLength);
    textNode.replaceWith(...highlightedNodes);
  });
  normalize.call(html);
  body && normalize.call(body);
  if (options.inplace) {
    return html;
  }
  if (options.returnDom) {
    if (options.returnDomFragment) {
      const returnedNode = createDocumentFragment.call(body.ownerDocument);
      while (body.firstChild) {
        returnedNode.appendChild(body.firstChild);
      }
      return returnedNode;
    }
    return body;
  }
  return options.returnWholeBody ? body.outerHTML : body.innerHTML;
}

exports.isSpecificWord = isSpecificWord;
exports.prehighlight = prehighlight;
